<?php
/**
* @package: phpBB 3.0.8 :: The Ultimate server error handler -> root
* @version: $Id: server_error.php, v 1.0.0 2011/03/13 11:03:13 leviatan21 Exp $
* @copyright: leviatan21 < info@mssti.com > (Gabriel) http://www.mssti.com/phpbb3/
* @license: http://opensource.org/licenses/gpl-license.php GNU Public License 
* @author: leviatan21 - http://www.phpbb.com/community/memberlist.php?mode=viewprofile&u=345763
*
* http://www.askapache.com/htaccess/apache-status-code-headers-errordocument.html
* http://www.askapache.com/htaccess/htaccess.html
**/

/**
* @ignore
**/
$error_numbers = array(
	400, 401, 403, 404, 405, 406, 407, 408, 409, 410, 411, 412, 413, 414, 415, 416, 417, 418, 422, 423, 424, 425, 426, 449,
	500, 501, 502, 503, 504, 505, 506, 507, 509, 510
);

// Default config vars
$error_config = array
(
	'PHPBB'	=> array(
		'enable'	=> true,
		'mode'		=> array(
			'msg_handler'	=> false,
			'style'			=> array('simple' => false),
		),
		'redirect'	=> false,
	),
	'HTML'	=> array(
		'enable'	=> true,
		'mode'		=> array(
			'msg_handler'	=> true,
			'style'			=> array('simple' => false),
		),
		'board_contact'		=> '',
	),
);

// Main handler
$phpbb_root_path = (defined('PHPBB_ROOT_PATH')) ? PHPBB_ROOT_PATH : './';
$phpEx = substr(strrchr(__FILE__, '.'), 1);
if ($error_config['PHPBB']['enable'] && @file_exists($phpbb_root_path . 'common.' . $phpEx))
{
	define('IN_PHPBB', true);
	include($phpbb_root_path . 'common.' . $phpEx);

	// Start session management
	$user->session_begin();
	$auth->acl($user->data);
	$user->setup();

	$user->lang['MSSTI_LINK'] = '<strong>Ultimate server error handler</strong> by <a href="http://www.mssti.com/phpbb3" onclick="window.open(this.href);return false;">.:: MSSTI ::.</a>';

	// Display error using phpbb
	servererror_phpbb_display();
}
else
{
	servererror_html_display();
}

/**
* Display error using phpbb
**/
function servererror_phpbb_display()
{
	global $user, $template;
	global $config, $phpbb_root_path, $phpEx;
	global $error_config;

	// Get the error number
	$errno = (int) $_SERVER['REDIRECT_STATUS'];
	if (!$errno)
	{
		$errno = request_var('error', 0);
	}

	// Get the error string for the error number 
	$error_ary		= servererror_phpbb_language($errno);
	$error_status	= (isset($error_ary['status'])) ? $error_ary['status'] : '';
	$error_text		= (isset($error_ary['message'])) ? $error_ary['message'] : '';
	$error_text 	= $error_text . (($errno) ? " ($errno)" : '');
	$error_title	= (($error_status) ? $error_status : 'ERROR') . (($errno) ? " ($errno)" : '');
	$request_url	= $_SERVER['REQUEST_URI'];

	// Outputs correct status line header
	if ($error_status != '')
	{
		servererror_send_status_line($errno, $error_status);
	}

	$template_body = 'server_error_body.html';
	// Check if the template is available
	if (!$error_config['PHPBB']['mode']['msg_handler'] && 
		(
			!file_exists($phpbb_root_path . 'styles/' . $user->theme['template_path'] . "/template/$template_body")
			&& 
			!file_exists($phpbb_root_path . 'styles/' . $user->theme['template_inherit_path'] . "/template/$template_body")
		)
	)
	{
		$error_config['PHPBB']['mode']['msg_handler'] = true;
	}

	// Add my link to the output page footer into the copyright notes ;)
	$user->lang['TRANSLATION_INFO'] = (!empty($user->lang['TRANSLATION_INFO'])) ? $user->lang['TRANSLATION_INFO'] . '<br />' . $user->lang['MSSTI_LINK'] : $user->lang['MSSTI_LINK'];

	// Use the phpbb message handler error mode ?
	if (!$error_config['PHPBB']['mode']['msg_handler'])
	{
		page_header($user->lang['GENERAL_ERROR']);

		$template->assign_vars(array(
			'SIMPLE_STYLE'		=> $error_config['PHPBB']['mode']['style']['simple'],
			'TITLE_ERROR'		=> $error_title,
			'FILE_ERROR'		=> $request_url,
			'MSG_ERROR'			=> $error_text,
			'U_CONTACT_ERROR'	=> (!empty($config['board_contact'])) ? '<p>Please notify the board administrator or webmaster: <a href="mailto:' . $config['board_contact'] . '">' . $config['board_contact'] . '</a></p>' : '',
			'U_RETURN_INDEX'	=> sprintf($user->lang['RETURN_INDEX'], '<a href="' . append_sid("{$phpbb_root_path}index.$phpEx") . '">', '</a>'),
			'META'				=> ($error_config['PHPBB']['redirect']) ? '<meta http-equiv="refresh" content="' . 5 . ';url=' . append_sid("{$phpbb_root_path}index.$phpEx") . '" />' : '',
		));

		$template->set_filenames(array(
			'body' => $template_body,
		));

		page_footer();
	}
	// Yes, use the phpbb message handler error mode (default)
	else
	{
		global $msg_title;
		$msg_title = $error_title;

		// Add the back to the index link
		$u_back = sprintf($user->lang['RETURN_INDEX'], '<a href="' . append_sid("{$phpbb_root_path}index.$phpEx") . '">', '</a>');

		// Add the requested url to the message
		$message = $error_text . "<br /><br />(" . $request_url . ")<br /><br />$u_back";
		trigger_error($message, E_USER_ERROR);
	}
}

/**
* Display error using html
**/
function servererror_html_display()
{
	global $error_config;

	// Get the error number
	$errno = $_SERVER['REDIRECT_STATUS'];
	if (!$errno)
	{
		$errno = (isset($_GET['error'])) ? intval($_GET['error']) : (isset($_POST['error']) ? intval($_POST['error']) : 0);
	}

	// Get the error string for the error number 
	$error_ary		= servererror_phpbb_language($errno);
	$error_status	= (isset($error_ary['status'])) ? $error_ary['status'] : '';
	$error_text 	= (isset($error_ary['message'])) ? $error_ary['message'] : '';
	$error_text 	= $error_text . (($errno) ? " ($errno)" : '');
	$board_contact	= $error_config['HTML']['board_contact'];
	$request_url	= $_SERVER['REQUEST_URI'];

	// Add the back to the index link
	$u_back = '<a href="./index.php">Home</a>';

	// Add the requested url to the message
	$message = $error_text . "<br /><br />(" . $request_url . ")<br /><br />";

	$title = (($error_status != '') ? $error_status : 'ERROR') . " ($errno)";

	// Outputs correct status line header
	if ($error_status != '')
	{
		servererror_send_status_line($errno, $error_status);
	}

	// Use the emulate phpbb message handler error mode ?
	if ($error_config['HTML']['mode']['msg_handler'])
	{
		$custom_style = get_custom_style('msg_handler', $title, $message, $u_back, $board_contact);

		echo $custom_style;

		@ob_flush();
	}
	else
	{
		$custom_style = get_custom_style($error_config['HTML']['mode']['style']['simple'], $title, $message, $u_back, $board_contact);

		echo $custom_style;

		@ob_flush();
	}
}

/**
* Fallback options for missing language files
* based off $user->setup()
*
* @param integer	$errno
* @param string		$lang_file
* @return string	language text
**/
function servererror_phpbb_language($errno = 0, $lang_file = 'mods/server_error')
{
	global $phpEx, $config, $user;

	$error = '';

//	$user_language_filename		= $user->lang_path . $user->lang_name . '/' . $lang_file . '.' . $phpEx;
//	$config_language_filename	= $user->lang_path . basename($config['default_lang']) . '/' . $lang_file . '.' . $phpEx;
//	$english_language_filename	= $user->lang_path . 'en' . '/' . $lang_file . '.' . $phpEx;

//	$language_error = (!file_exists($user_language_filename) || !file_exists($config_language_filename) || !file_exists($english_language_filename)) ? true : false;

//	if (!$language_error)
//	{
//		$user->add_lang($lang_file);
//	}

	if (isset($user->lang[$errno]))
	{
		$error = $user->lang[$errno];
	}
	else
	{
		$error = servererror_html_language($errno);
	}

	return $error;
}

/**
* Default language strings
*
* @param integer	$errno
* @return string	language text
**/
function servererror_html_language($errno = 0)
{
	$error = array('status' => '', 'message' => '');

	switch($errno)
	{
	/** NOT USED
		// 300 Solicitud del cliente vuelto a dirigir - INICIO
		case 300:
			$error = array(
				'status'	=> 'Multiple Choices',
			//	'message'	=> 'Indicates multiple options for the resource that the client may follow. It, for instance, could be used to present different format options for video, list files with different extensions, or word sense disambiguation.',
				'message'	=> 'Indica múltiples opciones para el recurso que el cliente puede seguir. Es, por ejemplo, podría ser utilizado para presentar las diferentes opciones de formato de vídeo, archivos de lista con diferentes extensiones, o el sentido ambiguo de la palabra.<br />(Opciones múltiples)',
			);
		break;	
		case 301:
			$error = array(
				'status'	=> 'Moved Permanently',
			//	'message'	=> 'This and all future requests should be directed to the given URI.',
				'message'	=> 'Esta y todas las futuras solicitudes deben ser dirigidas a otra URL<br />(Movido permanentemente)',
			);
		break;	
		case 302:
			$error = array(
				'status'	=> 'Moved Temporarily',
			//	'message'	=> 'Your Web server thinks that your URL has been temporarily redirected to another URL. The client system is expected to immediately retry the alternate URL.',
				'message'	=> 'El servidor web piensa que la URL se ha redirigido temporalmente a otra URL. El sistema cliente se espera que vuelva a intentar de inmediato la dirección alternativa - (Movido temporalmente)',
			);
		break;	
		case 303:
			$error = array(
				'status'	=> 'See Other',
			//	'message'	=> 'The response to the request can be found under another URI using a GET method. When received in response to a PUT, it should be assumed that the server has received the data and the redirect should be issued with a separate GET message.',
				'message'	=> 'La respuesta a la solicitud puede ser encontrado en otra URL utilizando un método GET. Cuando se reciban en respuesta a un PUT, que es de suponer que el servidor ha recibido los datos y la redirección debe ser emitido con un mensaje GET por separado.<br />(See Other)',
			);
		break;
		case 304:
			$error = array(
				'status'	=> 'Not Modified',
			//	'message'	=> 'The resource has not been modified since last requested.',
				'message'	=> 'El recurso no ha sido modificado desde la última solicitud<br />(Not Modified)',
			);
		break;
		case 305:
			$error = array(
				'status'	=> 'Use Proxy',
			//	'message'	=> 'The requested resource has to be reached through a proxy, which will be specified in the response.',
				'message'	=> 'El recurso solicitado tiene que ser alcanzado a través de un proxy, que se especificarán en la respuesta.<br />(Use Proxy)',
			);
		break;
	//	case 306:
			$error = array(
				'status'	=> 'Switch Proxy',
			//	'message'	=> 'Error no longer used.',
				'message'	=> 'El error ya no se utiliza - (Switch Proxy)',
			);
	//	break;
		case 307:
			$error = array(
				'status'	=> 'Temporary Redirec',
			//	'message'	=> 'The request should be repeated with another URI, but future requests can still use the original URI. For instance, a POST request must be repeated using another POST request.',
				'message'	=> 'La petición debe repetirse con otra URI, pero las futuras solicitudes de seguir utilizando el URI original. Por ejemplo, una petición POST debe repetirse con otra petición POST.br />(Redirección temporal)',
			);
		break;
		// 300 Solicitud del cliente vuelto a dirigir - FIN
	**/
		// 400 Errores de solicitaciones del cliente - INICIO
		case 400:
			$error = array(
				'status'	=> 'Bad Request',
			//	'message'	=> 'The request contains bad syntax or cannot be fulfilled.',
				'message'	=> 'La solicitud contiene sintaxis errónea o no puede cumplirse.<br />(Solicitud incorrecta)',
			);
		break;
		case 401:
			$error = array(
				'status'	=> 'Authorization Required',
			//	'message'	=> 'The request requires user authentication. The response MUST include a WWW-Authenticate header field containing a challenge applicable to the requested resource.',
				'message'	=> 'La solicitud requiere autenticación de usuario. La respuesta debe incluir un campo de encabezado WWW-Authenticate que contiene un reto aplicable al recurso solicitado.<br />(No autorizado)',
			);
		break;
	//	case 402:
	//		$error = array(
	//			'status'	=> 'Payment Required',
	//			'message'	=> 'Error no longer used',
	//			'message'	=> 'El error ya no se utiliza<br />(Pago requerido)',
	//		);
	//	break;
		case 403:
			$error = array(
				'status'	=> 'Forbidden',
			//	'message'	=> 'The request was a legal request, but the server is refusing to respond to it',
				'message'	=> 'La solicitud fue una solicitud legal, pero el servidor se niega a responder a ella.<br />(Prohibido)',
			);
		break;
		case 404:
			$error = array(
				'status'	=> 'Not Found',
			//	'message'	=> 'The requested resource could not be found but may be available again in the future. Subsequent requests by the client are permissible.',
				'message'	=> 'El recurso solicitado no se pudo encontrar, pero pueden estar disponibles de nuevo en el futuro. Las peticiones posteriores por parte del cliente son permisibles.<br />(No encontrado)',
			);
		break;
		case 405:
			$error = array(
				'status'	=> 'Method Not Allowed',
			//	'message'	=> 'A request was made of a resource using a request method not supported by that resource.',
				'message'	=> 'La solicitud fue hecha de un recurso mediante un método de solicitud no es compatible con ese recurso.<br />(Método no permitido)',
			);
		break;
		case 406:
			$error = array(
				'status'	=> 'Not Acceptable',
			//	'message'	=> 'The requested resource is only capable of generating content not acceptable according to the Accept headers sent in the request.',
				'message'	=> 'El servidor web detecta que los datos que quiere enviar no son aceptables para el cliente.<br />(No Aceptable (codificación))',
			);
		break;
		case 407:
			$error = array(
				'status'	=> 'Proxy Authentication Required',
			//	'message'	=> 'The client must first authenticate itself with the proxy.',
				'message'	=> 'El acceso al recurso de la URL requiere el previo uso de un servidor proxy que necesita alguna autenticación que no se ha provisto.<br />(Requiere autenticación de proxy)',
			);
		break;
		case 408:
			$error = array(
				'status'	=> 'Request Time-out',
			//	'message'	=> 'The server timed out waiting for the request.',
				'message'	=> 'El servidor ha agotado el tiempo de espera para la solicitud.<br />(Tiempo de espera agotado)',
			);
		break;
		case 409:
			$error = array(
				'status'	=> 'Conflict',
			//	'message'	=> 'The request could not be processed because of conflict in the request.',
				'message'	=> 'La petición no podía ser procesado debido a los conflictos en la solicitud.<br />(Sulicitud conflictiva)',
			);
		break;
		case 410:
			$error = array(
				'status'	=> 'Gone',
			//	'message'	=> 'The resource requested is no longer available and will not be available again. The client should not request the resource again in the future. Search engines should remove the resource from their indexes..',
				'message'	=> 'El recurso solicitado ya no está disponible y no volverá a estar disponible. El cliente no debe solicitar el recurso de nuevo en el futuro. Los motores de búsqueda debe quitar el recurso de sus índices.<br />(Ya no existe)',
			);
		break;
		case 411:
			$error = array(
				'status'	=> 'Length Required',
			//	'message'	=> 'The request did not specify the length of its content, which is required by the requested resource.',
				'message'	=> 'El servidor se niega a procesar una solicitud debido a una longitud de contenido que no se ha especificado.<br />(Contenido longitud requerida)',
			);
		break;
		case 412:
			$error = array(
				'status'	=> 'Precondition Failed',
			//	'message'	=> 'The server does not meet one of the preconditions that the requester put on the request.',
				'message'	=> 'El servidor no cumple uno de los requisitos que se ha solicitado.<br />(Fallo de la condición previa)',
			);
		break;
		case 413:
			$error = array(
				'status'	=> 'Request Entity Too Large',
			//	'message'	=> 'The request is larger than the server is willing or able to process.',
				'message'	=> 'La solicitud es mayor que el servidor está dispuesto o es capaz de procesar.<br />(Petición demasiado larga)',
			);
		break;
		case 414:
			$error = array(
				'status'	=> 'Request-URI Too Large',
			//	'message'	=> 'The URI provided was too long for the server to process.',
				'message'	=> 'La URL requerida es demasiado larga.<br />(URL solicitada demasiado larga)',
			);
		case 415:
			$error = array(
				'status'	=> 'Unsupported Media Type',
			//	'message'	=> 'The request did not specify any media types that the server or resource supports.',
				'message'	=> 'La solicitud no se especifica ningún tipo de medios que admite el servidor o recurso.<br />(Tipo de medios no admitidos)',
			);
		break;
		case 416:
			$error = array(
				'status'	=> 'Requested Range Not Satisfiable',
			//	'message'	=> 'The client has asked for a portion of the file, but the server cannot supply that portion.',
				'message'	=> 'El cliente ha solicitado una parte del archivo, pero el servidor no puede proporcionar esa parte.<br />(Rango solicitado no puede ser satisfecho)',
			);
		break;
		case 417:
			$error = array(
				'status'	=> 'Expectation Failed',
			//	'message'	=> 'The server cannot meet the requirements of the Expect request-header field.',
				'message'	=> 'El servidor cree que el flujo de datos HTTP enviado por el cliente contiene una solicitud que no puede ser satisfecha<br />(Falló expectativa)',
			);
		break;
		case 418:
			$error = array(
				'status'	=> 'I’m a teapot',
			//	'message'	=> 'The HTCPCP server is a teapot. The responding entity MAY be short and stout. Defined by the April Fools specification RFC 2324. See <b>Hyper Text Coffee Pot Control Protocol</b> for more information.',
				'message'	=> 'El servidor HTCPCP es una tetera. The responding entity MAY be short and stout. La respuesta entidad puede ser corto y grueso. Definido por la especificación de los Inocentes RFC 2324. Ver <b>Hyper Text Coffee Pot Control Protocol</b> para obtener más información.<br />(Soy una tetera)',
			);
		break;
		case 422:
			$error = array(
				'status'	=> 'Unprocessable Entity',
			//	'message'	=> 'The request was well-formed but was unable to be followed due to semantic errors.',
				'message'	=> 'La solicitud fue bien formada, pero no pudo seguir debido a los errores semánticos.<br />(Entidad no procesable)',
			);
		break;
		case 423:
			$error = array(
				'status'	=> 'Locked',
			//	'message'	=> 'The resource that is being accessed is locked.',
				'message'	=> 'El recurso al que se quire acceder está bloqueado.<br />(Entidad no procesable)',
			);
		break;
		case 424:
			$error = array(
				'status'	=> 'Failed Dependency',
			//	'message'	=> 'The request failed due to failure of a previous request.',
				'message'	=> 'La solicitud ha fallado debido a la falta de una solicitud anterior.<br />(Entidad no procesable)'
			);
		break;
		case 425:
			$error = array(
				'status'	=> 'Locked',
			//	'message'	=> 'Defined in drafts of “WebDav Advanced Collections”, but not present in “Web Distributed Authoring and Versioning (WebDAV) Ordered Collections Protocol” (RFC 3648).',
				'message'	=> 'Definido en proyectos de “WebDav Advanced Collections”, pero no están presentes en “Web Distributed Authoring and Versioning (WebDAV) Ordered Collections Protocol” (RFC 3648).<br />(Colección desordenada)',
			);
		break;
		case 426:
			$error = array(
				'status'	=> 'Upgrade Required',
			//	'message'	=> 'The client should switch to TLS/1.0.',
				'message'	=> 'El cliente debe cambiar a TLS/1.0.<br />(Actualización necesaria)'
			);
		break;
		case 449:
			$error = array(
				'status'	=> 'Retry With',
			//	'message'	=> 'A Microsoft extension. The request should be retried after doing the appropriate action.',
				'message'	=> 'Una extensión de Microsoft. La solicitud debe ser juzgado de nuevo después de hacer la acción apropiada.<br />(Reintentar)'
			);
		break;
		// 400 Errores de solicitaciones del cliente - FIN

		// 500 Error del Servidor - INICIO
		case 500:
			$error = array(
				'status'	=> 'Internal Server Error',
			//	'message'	=> 'The server encountered an unexpected condition which prevented it from fulfilling the request. There is no more specific message.',
				'message'	=> 'El servidor web encontró una condición inesperada que le impidió completar la solicitud del cliente para acceder a la URL requerida.<br />(Error interno del servidor)'
			);
		break;
		case 501:
			$error = array(
				'status'	=> 'Method Not Implemented',
			//	'message'	=> 'The server either does not recognise the request method, or it lacks the ability to fulfil the request.',
				'message'	=> 'El servidor no reconoce el método solicitado, o carece de la capacidad para cumplir con la solicitud.<br />(Not Implemented Error)'
			);
		break;
		case 502:
			$error = array(
				'status'	=> 'Bad Gateway',
			//	'message'	=> 'The server was acting as a gateway or proxy and received an invalid response from the upstream server.',
				'message'	=> 'El servidor está actuando como una puerta de enlace o proxy para acceder la URL requerida. Este servidor recibió una respuesta inválida desde un servidor de flujo ascendente al cual accedió para completar la solicitud.<br />(Puerta de enlace incorrecta)'
			);
		break;
		case 503:
			$error = array(
				'status'	=> 'Service Temporarily Unavailable',
			//	'message'	=> 'The server is currently unavailable (because it is overloaded or down for maintenance). Generally, this is a temporary state.',
				'message'	=> 'El servidor no está disponible (porque está sobrecargado o fuera de servicio por mantenimiento). En general, este es un estado temporal.<br />(Servidor no disponible)'
			);
		break;
		case 504:
			$error = array(
				'status'	=> 'Gateway Time-out',
			//	'message'	=> 'The server was acting as a gateway or proxy and did not receive a timely request from the upstream server.',
				'message'	=> 'El servidor está actuando como una puerta de enlace o proxy, no recibió una solicitud puntual del servidor ascendente.<br />(Tiempo de espera de gateway superado)',
			);
		break;
		case 505:
			$error = array(
				'status'	=> 'HTTP Version Not Supported',
			//	'message'	=> 'The server does not support the HTTP protocol version used in the request.',
				'message'	=> 'El servidor no es compatible con la versión del protocolo HTTP que se utiliza en la solicitud.<br />(HTTP Version Not Supported)'
			);
		break;
		case 506:
			$error = array(
				'status'	=> 'Variant Also Negotiates',
			//	'message'	=> 'Transparent content negotiation for the request, results in a circular reference (RFC 2295).',
				'message'	=> 'La negociación de contenido transparente para la solicitud, los resultados en una referencia circular (RFC 2295).<br />(Variante también negocia)'
			);
		break;
		case 507:
			$error = array(
				'status'	=> 'Insufficient Storage',
			//	'message'	=> 'The method could not be performed on the resource because the server is unable to store the representation needed to successfully complete the request. There is insufficient free space left in your storage allocation.',
				'message'	=> 'El método no se pudo realizar en el recurso porque el servidor es incapaz de almacenar la representación necesaria para completar con éxito la solicitud. No hay suficiente espacio libre que queda en su asignación de almacenamiento.<br />(insuficiente de almacenamiento)',
			);
		break;
		case 509:
			$error = array(
				'status'	=> 'unused',
			//	'message'	=> '(Apache bw/limited extension) - This status code, while used by many servers, is not specified in any RFCs.<br />(Bandwidth Limit Exceeded)',
				'message'	=> '(Pc Apache/extensión limitada) - Este código de estado, mientras utilizado por muchos servidores, no se especifica en ningún RFC.<br />(Se ha superado el ancho de banda)'
			);
		break;
		case 510:
			$error = array(
				'status'	=> 'unused',
			//	'message'	=> 'Further extensions to the request are required for the server to fulfil it (RFC 2774).<br />(Not Extended)',
				'message'	=> 'Extensiones de respuesta a la petición se requieren para el servidor para que pueda desempeñar (RFC 2774)<br />(No extendido)',
			);
		break;
		// 500 Error del Servidor - FIN

		default:
			$error = array(
				'status' => '',
			//	'message' => 'An UNIDENTIFIED error occurs',
				'message' => 'Se ha producido un error NO IDENTIFICADO !',
			);
		break;
	}

	return $error;
}

/**
* Enter description here...
*
* @param string 	$mode
* @param string 	$error_title
* @param string 	$error_txt
* @param string 	$error_uback
* @return string	html code
**/
function get_custom_style($mode, $error_title = '', $error_txt = '', $error_uback = '', $board_contact = '')
{
	if ($mode === 'msg_handler')
	{
		$html  = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">' .
		$html .= '<html xmlns="http://www.w3.org/1999/xhtml" dir="ltr">';
		$html .= '<head>';
		$html .= '	<meta http-equiv="content-type" content="text/html; charset=utf-8" />';
		$html .= '	<title>{TITLE}</title>';
		$html .= '	<style type="text/css">' . "\n";
		$html .= '	/* <![CDATA[ */' . "\n";
		$html .= '	* { margin: 0; padding: 0; } html { font-size: 100%; height: 100%; margin-bottom: 1px; background-color: #E4EDF0; } body { font-family: "Lucida Grande", Verdana, Helvetica, Arial, sans-serif; color: #536482; background: #E4EDF0; font-size: 62.5%; margin: 0; } ';
		$html .= '	a:link, a:active, a:visited { color: #006699; text-decoration: none; } a:hover { color: #DD6900; text-decoration: underline; } ';
		$html .= '	#wrap { padding: 0 20px 15px 20px; min-width: 615px; } #page-header { text-align: right; height: 40px; } #page-footer { clear: both; font-size: 1em; text-align: center; } ';
		$html .= '	.panel { margin: 4px 0; background-color: #FFFFFF; border: solid 1px  #A9B8C2; } ';
		$html .= '	#errorpage #page-header a { font-weight: bold; line-height: 6em; } #errorpage #content { padding: 10px; } #errorpage #content h1 { line-height: 1.2em; margin-bottom: 0; color: #DF075C; } ';
		$html .= '	#errorpage #content div { margin-top: 20px; margin-bottom: 5px; border-bottom: 1px solid #CCCCCC; padding-bottom: 5px; color: #333333; font: bold 1.2em "Lucida Grande", Arial, Helvetica, sans-serif; text-decoration: none; line-height: 120%; text-align: left; } ';
		$html .= "	\n" . '/* ]]> */' . "\n";
		$html .= '	</style>' . "\n";
		$html .= '</head>' . "\n";
		$html .= '<body id="errorpage">' . "\n";
		$html .= '<div id="wrap">' . "\n";
		$html .= '	<div id="page-header">' . "\n";
		$html .= '		{U_RETURN_INDEX}' . "\n";
		$html .= '	</div>' . "\n";
		$html .= '	<div id="acp">' . "\n";
		$html .= '	<div class="panel">' . "\n";
		$html .= '		<div id="content">' . "\n";
		$html .= '			<h1>{TITLE}</h1>' . "\n";
		$html .= '			<div>{MSG_ERROR}' . "\n";
		$html .= '				<p>{U_RETURN_INDEX}</p>' . "\n";
		$html .= '			</div>' . "\n";
		$html .= '		{NOTIFY_ADMIN_EMAIL}' . "\n";
		$html .= '		</div>' . "\n";
		$html .= '	</div>' . "\n";
		$html .= '	</div>' . "\n";
		$html .= '	<div id="page-footer">' . "\n";
		$html .= '		Powered by phpBB &copy; 2000, 2002, 2005, 2007 <a href="http://www.phpbb.com/">phpBB Group</a>' . "\n";
		$html .= '		<br /><strong>Ultimate server error handler</strong> by <a href="http://www.mssti.com/phpbb3" onclick="window.open(this.href);return false;">.:: MSSTI ::.</a>' . "\n";
		$html .= '	</div>' . "\n";
		$html .= '</div>' . "\n";
		$html .= '</body>' . "\n";
		$html .= '</html>';
	}
	else if ($mode == false)
	{
		$html .= '<html>' . "\n";
		$html .= '<head>' . "\n";
		$html .= '	<title>{TITLE}</title>' . "\n";
		$html .= '	<style type="text/css">' . "\n";
		$html .= '	/* <![CDATA[ */' . "\n";
		$html .= '	* { margin: 0; padding: 0; }' . "\n";
		$html .= '	html { font-size: 100%; height: 100%; margin-bottom: 1px; background-color: #ffffff; }' . "\n";
		$html .= '	body { font-family: "Lucida Grande", Verdana, Helvetica, Arial, sans-serif; color: #000000; background: #cccccc; font-size: 62.5%; margin: 0; border: 1px solid #000000; margin: 1em; }' . "\n";
		$html .= '	a:link, a:active, a:visited { color: #006699; text-decoration: none; } a:hover { color: #ff0000; text-decoration: underline; }' . "\n";
		$html .= '	#wrap { padding: 2em 1em; min-width: 615px; text-align: left; border: 1px solid gray; font: bold 1.2em "Lucida Grande", Arial, Helvetica, sans-serif; text-decoration: none; line-height: 120%; }' . "\n";
		$html .= '	h1 { line-height: 1.2em; margin-bottom: 1em; color: #000000; border-bottom: 1px solid #000000; text-align: left; }' . "\n";
		$html .= '	.notify { border-top: 1px solid #000000; margin-top: 1em; padding-top: 1em; text-align: left; }' . "\n";
		$html .= '	#page-footer { clear: both; font-size: 1em; text-align: center; }' . "\n";
		$html .= "	\n" . '/* ]]> */' . "\n";
		$html .= '	</style>' . "\n";
		$html .= '</head>' . "\n";
		$html .= '<body id="errorpage">' . "\n";
		$html .= '	<div id="wrap">' . "\n";
		$html .= '		<h1>{TITLE}</h1>' . "\n";
		$html .= '		<p>{MSG_ERROR}</p>' . "\n";
		$html .= '		<p>{U_RETURN_INDEX}</p>' . "\n";
		$html .= '		{NOTIFY_ADMIN_EMAIL}' . "\n";
		$html .= '	</div>' . "\n";
		$html .= '	<div id="page-footer">' . "\n";
		$html .= '		Powered by phpBB &copy; 2000, 2002, 2005, 2007 <a href="http://www.phpbb.com/">phpBB Group</a>' . "\n";
		$html .= '		<br /><b>Ultimate server error handler</b> by <a href="http://www.mssti.com/phpbb3" onclick="window.open(this.href);return false;">.:: MSSTI ::.</a>' . "\n";
		$html .= '	</div>' . "\n";
		$html .= '</body>' . "\n";
		$html .= '</html>';
	}
	else
	{
		$html .= '<html>' . "\n";
		$html .= '<head>' . "\n";
		$html .= '	<title>{TITLE}</title>' . "\n";
		$html .= '</head>' . "\n";
		$html .= '<body>' . "\n";
		$html .= '	<h1>{TITLE}</h1>' . "\n";
		$html .= '	<p>{MSG_ERROR}</p>' . "\n";
		$html .= '	<p>{U_RETURN_INDEX}</p>' . "\n";
		$html .= '	{NOTIFY_ADMIN_EMAIL}' . "\n";
		$html .= '</body>' . "\n";
		$html .= '</html>';
	}
	// Parse Theme Data
	$replace = array(
		'{S_USER_LANG}'			=> get_custom_language(),
		'{TITLE}'				=> $error_title,
		'{MSG_ERROR}'			=> $error_txt,
		'{U_RETURN_INDEX}'		=> $error_uback,
		'{NOTIFY_ADMIN_EMAIL}'	=> ($board_contact) ? '<p class="notify">Please notify the board administrator or webmaster: <a href="mailto:' . $board_contact . '">' . $board_contact . '</a></p>' : '',
	);

	$html = str_replace(array_keys($replace), array_values($replace), $html);

	return $html;
}

/**
* We try to guess his/her language first by obtaining the browser language
**/
function get_custom_language()
{
	$accept_lang = 'en';

	if (isset($_SERVER['HTTP_ACCEPT_LANGUAGE']))
	{
		$accept_lang_ary = explode(',', $_SERVER['HTTP_ACCEPT_LANGUAGE']);

		foreach ($accept_lang_ary as $accept_lang)
		{
			// Set correct format ... guess full xx_YY form
			$accept_lang = substr($accept_lang, 0, 2) . '-' . substr($accept_lang, 3, 2);
			$accept_lang = basename($accept_lang);

			if (preg_match('/^[a-zA-Z]{2,}\-[a-zA-Z]{2,}$/', $accept_lang))
			{
				break;
			}
			else
			{
				// No match on xx-YY so try xx
				$accept_lang = substr($accept_lang, 0, 2);
				$accept_lang = basename($accept_lang);

				if (preg_match('#(?:[^\w]|$)#i', $accept_lang))
				{
					break;
				}
			}
		}
	}

	return $accept_lang;
}

/**
* Outputs correct status line header.
*
* Depending on php sapi one of the two following forms is used:
*
* Status: 404 Not Found
*
* HTTP/1.x 404 Not Found
*
* HTTP version is taken from HTTP_VERSION environment variable,
* and defaults to 1.0.
*
* Sample usage:
*
* send_status_line(404, 'Not Found');
*
* @param int $code HTTP status code
* @param string $message Message for the status code
* @return void
*/
function servererror_send_status_line($code, $message)
{
	if (substr(strtolower(@php_sapi_name()), 0, 3) === 'cgi')
	{
		// in theory, we shouldn't need that due to php doing it. Reality offers a differing opinion, though
		header("Status: $code $message", true, $code);
	}
	else
	{
		if (isset($_SERVER['HTTP_VERSION']))
		{
			$version = $_SERVER['HTTP_VERSION'];
		}
		else
		{
			$version = 'HTTP/1.0';
		}
		header("$version $code $message", true, $code);
	}
}

?>